@@ -13,14 +13,17 @@ middleware here, or combine a Django application with an application of another
 framework.
 
 """
-import os
+from __future__ import absolute_import, unicode_literals
 
-os.environ.setdefault("DJANGO_SETTINGS_MODULE", "proj.settings")
+import os
 
 # This application object is used by any WSGI server configured to use this
 # file. This includes Django's development server, if the WSGI_APPLICATION
 # setting points here.
 from django.core.wsgi import get_wsgi_application  # noqa
+
+os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'proj.settings')
+
 application = get_wsgi_application()
 
 # Apply WSGI middleware here.
