@@ -1,7 +1,6 @@
 from __future__ import absolute_import, unicode_literals
 
 import os
-from functools import wraps
 
 import pytest
 
@@ -18,25 +17,11 @@ TEST_BACKEND = os.environ.get('TEST_BACKEND', 'redis://')
 __all__ = (
     'celery_app',
     'celery_session_worker',
-    'flaky',
     'get_active_redis_channels',
     'get_redis_connection',
 )
 
 
-def flaky(fun):
-    @wraps(fun)
-    def _inner(*args, **kwargs):
-        for i in reversed(range(3)):
-            try:
-                return fun(*args, **kwargs)
-            except Exception:
-                if not i:
-                    raise
-    _inner.__wrapped__ = fun
-    return _inner
-
-
 def get_redis_connection():
     from redis import StrictRedis
     return StrictRedis(host=os.environ.get('REDIS_HOST'))
@@ -50,7 +35,12 @@ def get_active_redis_channels():
 def celery_config():
     return {
         'broker_url': TEST_BROKER,
-        'result_backend': TEST_BACKEND
+        'result_backend': TEST_BACKEND,
+        'cassandra_servers': ['localhost'],
+        'cassandra_keyspace': 'tests',
+        'cassandra_table': 'tests',
+        'cassandra_read_consistency': 'ONE',
+        'cassandra_write_consistency': 'ONE'
     }
 
 
