@@ -6,6 +6,7 @@ from pickle import dumps, loads
 import pytest
 from case import ANY, MagicMock, Mock, mock, patch, sentinel, skip
 from kombu.exceptions import EncodeError
+from pymongo.errors import ConfigurationError
 
 from celery import states, uuid
 from celery.backends.mongodb import InvalidDocument, MongoBackend
@@ -129,8 +130,9 @@ class test_MongoBackend:
 
         self.app.conf.mongodb_backend_settings = None
 
-        def mock_resolver(_, record_type):
-            if record_type == 'SRV':
+        def mock_resolver(_, rdtype, rdclass=None, lifetime=None, **kwargs):
+
+            if rdtype == 'SRV':
                 return [
                     SRV(0, 0, 0, 0, 27017, Name(labels=hostname))
                     for hostname in [
@@ -139,7 +141,7 @@ class test_MongoBackend:
                         b'mongo3.example.com'.split(b'.')
                     ]
                 ]
-            elif record_type == 'TXT':
+            elif rdtype == 'TXT':
                 return [TXT(0, 0, [b'replicaSet=rs0'])]
 
         dns_resolver_query.side_effect = mock_resolver
@@ -219,6 +221,42 @@ class test_MongoBackend:
             )
             assert sentinel.connection == connection
 
+    def test_get_connection_with_authmechanism(self):
+        with patch('pymongo.MongoClient') as mock_Connection:
+            self.app.conf.mongodb_backend_settings = None
+            uri = ('mongodb://'
+                   'celeryuser:celerypassword@'
+                   'localhost:27017/'
+                   'celerydatabase?authMechanism=SCRAM-SHA-256')
+            mb = MongoBackend(app=self.app, url=uri)
+            mock_Connection.return_value = sentinel.connection
+            connection = mb._get_connection()
+            mock_Connection.assert_called_once_with(
+                host=['localhost:27017'],
+                username='celeryuser',
+                password='celerypassword',
+                authmechanism='SCRAM-SHA-256',
+                **mb._prepare_client_options()
+            )
+            assert sentinel.connection == connection
+
+    def test_get_connection_with_authmechanism_no_username(self):
+        with patch('pymongo.MongoClient') as mock_Connection:
+            self.app.conf.mongodb_backend_settings = None
+            uri = ('mongodb://'
+                   'localhost:27017/'
+                   'celerydatabase?authMechanism=SCRAM-SHA-256')
+            mb = MongoBackend(app=self.app, url=uri)
+            mock_Connection.side_effect = ConfigurationError(
+                'SCRAM-SHA-256 requires a username.')
+            with pytest.raises(ConfigurationError):
+                mb._get_connection()
+            mock_Connection.assert_called_once_with(
+                host=['localhost:27017'],
+                authmechanism='SCRAM-SHA-256',
+                **mb._prepare_client_options()
+            )
+
     @patch('celery.backends.mongodb.MongoBackend._get_connection')
     def test_get_database_no_existing(self, mock_get_connection):
         # Should really check for combinations of these two, to be complete.
@@ -235,7 +273,7 @@ class test_MongoBackend:
         assert database is mock_database
         assert self.backend.__dict__['database'] is mock_database
         mock_database.authenticate.assert_called_once_with(
-            MONGODB_USER, MONGODB_PASSWORD)
+            MONGODB_USER, MONGODB_PASSWORD, source=self.backend.database_name)
 
     @patch('celery.backends.mongodb.MongoBackend._get_connection')
     def test_get_database_no_existing_no_auth(self, mock_get_connection):
@@ -269,10 +307,11 @@ class test_MongoBackend:
 
         mock_get_database.assert_called_once_with()
         mock_database.__getitem__.assert_called_once_with(MONGODB_COLLECTION)
-        mock_collection.save.assert_called_once_with(ANY)
+        mock_collection.replace_one.assert_called_once_with(ANY, ANY,
+                                                            upsert=True)
         assert sentinel.result == ret_val
 
-        mock_collection.save.side_effect = InvalidDocument()
+        mock_collection.replace_one.side_effect = InvalidDocument()
         with pytest.raises(EncodeError):
             self.backend._store_result(
                 sentinel.task_id, sentinel.result, sentinel.status)
@@ -295,11 +334,11 @@ class test_MongoBackend:
 
         mock_get_database.assert_called_once_with()
         mock_database.__getitem__.assert_called_once_with(MONGODB_COLLECTION)
-        parameters = mock_collection.save.call_args[0][0]
+        parameters = mock_collection.replace_one.call_args[0][1]
         assert parameters['parent_id'] == sentinel.parent_id
         assert sentinel.result == ret_val
 
-        mock_collection.save.side_effect = InvalidDocument()
+        mock_collection.replace_one.side_effect = InvalidDocument()
         with pytest.raises(EncodeError):
             self.backend._store_result(
                 sentinel.task_id, sentinel.result, sentinel.status)
@@ -358,7 +397,8 @@ class test_MongoBackend:
         mock_database.__getitem__.assert_called_once_with(
             MONGODB_GROUP_COLLECTION,
         )
-        mock_collection.save.assert_called_once_with(ANY)
+        mock_collection.replace_one.assert_called_once_with(ANY, ANY,
+                                                            upsert=True)
         assert res == ret_val
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
@@ -401,7 +441,7 @@ class test_MongoBackend:
         self.backend._delete_group(sentinel.taskset_id)
 
         mock_get_database.assert_called_once_with()
-        mock_collection.remove.assert_called_once_with(
+        mock_collection.delete_one.assert_called_once_with(
             {'_id': sentinel.taskset_id})
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
@@ -420,7 +460,7 @@ class test_MongoBackend:
         mock_get_database.assert_called_once_with()
         mock_database.__getitem__.assert_called_once_with(
             MONGODB_COLLECTION)
-        mock_collection.remove.assert_called_once_with(
+        mock_collection.delete_one.assert_called_once_with(
             {'_id': sentinel.task_id})
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
@@ -440,7 +480,7 @@ class test_MongoBackend:
         self.backend.cleanup()
 
         mock_get_database.assert_called_once_with()
-        mock_collection.remove.assert_called()
+        mock_collection.delete_many.assert_called()
 
     def test_get_database_authfailure(self):
         x = MongoBackend(app=self.app)
@@ -452,7 +492,8 @@ class test_MongoBackend:
         x.password = 'cere4l'
         with pytest.raises(ImproperlyConfigured):
             x._get_database()
-        db.authenticate.assert_called_with('jerry', 'cere4l')
+        db.authenticate.assert_called_with('jerry', 'cere4l',
+                                           source=x.database_name)
 
     def test_prepare_client_options(self):
         with patch('pymongo.version_tuple', new=(3, 0, 3)):
