@@ -677,3 +677,26 @@ class test_State:
         s = State(callback=callback)
         s.event({'type': 'worker-online'})
         assert scratch.get('recv')
+
+    def test_deepcopy(self):
+        import copy
+        s = State()
+        s.event({
+            'type': 'task-success',
+            'root_id': 'x',
+            'uuid': 'x',
+            'hostname': 'y',
+            'clock': 3,
+            'timestamp': time(),
+            'local_received': time(),
+        })
+        s.event({
+            'type': 'task-success',
+            'root_id': 'y',
+            'uuid': 'y',
+            'hostname': 'y',
+            'clock': 4,
+            'timestamp': time(),
+            'local_received': time(),
+        })
+        copy.deepcopy(s)
