@@ -224,6 +224,34 @@ class test_Request(RequestCase):
         assert self.get_request(
             self.add.s(2, 2).set(shadow='fooxyz')).name == 'fooxyz'
 
+    def test_args(self):
+        args = (2, 2)
+        assert self.get_request(
+            self.add.s(*args)).args == args
+
+    def test_kwargs(self):
+        kwargs = {'1': '2', '3': '4'}
+        assert self.get_request(
+            self.add.s(**kwargs)).kwargs == kwargs
+
+    def test_info_function(self):
+        import string
+        import random
+        kwargs = {}
+        for i in range(0, 2):
+            kwargs[str(i)] = ''.join(random.choice(string.ascii_lowercase) for i in range(1000))
+        assert self.get_request(
+            self.add.s(**kwargs)).info(safe=True).get('kwargs') == kwargs
+        assert self.get_request(
+            self.add.s(**kwargs)).info(safe=False).get('kwargs') == kwargs
+        args = []
+        for i in range(0, 2):
+            args.append(''.join(random.choice(string.ascii_lowercase) for i in range(1000)))
+        assert list(self.get_request(
+            self.add.s(*args)).info(safe=True).get('args')) == args
+        assert list(self.get_request(
+            self.add.s(*args)).info(safe=False).get('args')) == args
+
     def test_no_shadow_header(self):
         request = self.get_request(self.add.s(2, 2),
                                    exclude_headers=['shadow'])
@@ -607,6 +635,26 @@ class test_Request(RequestCase):
         job.on_failure(exc_info)
         assert self.mytask.backend.get_status(job.id) == states.PENDING
 
+    def test_on_failure_acks_late_reject_on_worker_lost_enabled(self):
+        try:
+            raise WorkerLostError()
+        except WorkerLostError:
+            exc_info = ExceptionInfo()
+        self.mytask.acks_late = True
+        self.mytask.reject_on_worker_lost = True
+
+        job = self.xRequest()
+        job.delivery_info['redelivered'] = False
+        job.on_failure(exc_info)
+
+        assert self.mytask.backend.get_status(job.id) == states.PENDING
+
+        job = self.xRequest()
+        job.delivery_info['redelivered'] = True
+        job.on_failure(exc_info)
+
+        assert self.mytask.backend.get_status(job.id) == states.FAILURE
+
     def test_on_failure_acks_late(self):
         job = self.xRequest()
         job.time_start = 1
@@ -621,6 +669,7 @@ class test_Request(RequestCase):
     def test_on_failure_acks_on_failure_or_timeout_disabled_for_task(self):
         job = self.xRequest()
         job.time_start = 1
+        job._on_reject = Mock()
         self.mytask.acks_late = True
         self.mytask.acks_on_failure_or_timeout = False
         try:
@@ -628,7 +677,9 @@ class test_Request(RequestCase):
         except KeyError:
             exc_info = ExceptionInfo()
             job.on_failure(exc_info)
-        assert job.acknowledged is False
+
+        assert job.acknowledged is True
+        job._on_reject.assert_called_with(req_logger, job.connection_errors, False)
 
     def test_on_failure_acks_on_failure_or_timeout_enabled_for_task(self):
         job = self.xRequest()
@@ -653,7 +704,9 @@ class test_Request(RequestCase):
         except KeyError:
             exc_info = ExceptionInfo()
             job.on_failure(exc_info)
-        assert job.acknowledged is False
+        assert job.acknowledged is True
+        job._on_reject.assert_called_with(req_logger, job.connection_errors,
+                                          False)
         self.app.conf.acks_on_failure_or_timeout = True
 
     def test_on_failure_acks_on_failure_or_timeout_enabled(self):
