@@ -4,16 +4,16 @@ from collections import defaultdict
 from contextlib import contextmanager
 
 import pytest
-from case import Mock, patch
+from case import ANY, Mock, patch
 from kombu.utils.limits import TokenBucket
 
-from celery import Task
+from celery import Task, signals
 from celery.exceptions import InvalidTaskError
 from celery.utils.time import rate
 from celery.worker import state
 from celery.worker.request import Request
 from celery.worker.strategy import default as default_strategy
-from celery.worker.strategy import proto1_to_proto2
+from celery.worker.strategy import proto1_to_proto2, hybrid_to_proto2
 
 
 class test_proto1_to_proto2:
@@ -167,6 +167,15 @@ class test_default_strategy_proto2:
             for callback in callbacks:
                 callback.assert_called_with(req)
 
+    def test_signal_task_received(self):
+        callback = Mock()
+        with self._context(self.add.s(2, 2)) as C:
+            signals.task_received.connect(callback)
+            C()
+            callback.assert_called_once_with(sender=C.consumer,
+                                             request=ANY,
+                                             signal=signals.task_received)
+
     def test_when_events_disabled(self):
         with self._context(self.add.s(2, 2), events=False) as C:
             C()
@@ -259,3 +268,25 @@ class test_custom_request_for_default_strategy(test_default_strategy_proto2):
             )
             task_message_handler(C.message, None, None, None, None)
             _MyRequest.assert_called()
+
+
+class test_hybrid_to_proto2:
+
+    def setup(self):
+        self.message = Mock(name='message')
+        self.body = {
+            'args': (1,),
+            'kwargs': {'foo': 'baz'},
+            'utc': False,
+            'taskset': '123',
+        }
+
+    def test_retries_default_value(self):
+        _, headers, _, _ = hybrid_to_proto2(self.message, self.body)
+        assert headers.get('retries') == 0
+
+    def test_retries_custom_value(self):
+        _custom_value = 3
+        self.body['retries'] = _custom_value
+        _, headers, _, _ = hybrid_to_proto2(self.message, self.body)
+        assert headers.get('retries') == _custom_value
\ No newline at end of file
