@@ -10,8 +10,9 @@ from threading import Event
 
 import pytest
 from amqp import ChannelError
-from case import Mock, patch, skip
+from case import Mock, mock, patch, skip
 from kombu import Connection
+from kombu.asynchronous import get_event_loop
 from kombu.common import QoS, ignore_errors
 from kombu.transport.base import Message
 from kombu.transport.memory import Transport
@@ -29,7 +30,7 @@ from celery.platforms import EX_FAILURE
 from celery.utils.nodenames import worker_direct
 from celery.utils.serialization import pickle
 from celery.utils.timer2 import Timer
-from celery.worker import components, consumer, state
+from celery.worker import autoscale, components, consumer, state
 from celery.worker import worker as worker_module
 from celery.worker.consumer import Consumer
 from celery.worker.pidbox import gPidbox
@@ -791,6 +792,110 @@ class test_WorkController(ConsumerCase):
         )
         assert worker.autoscaler
 
+    @pytest.mark.nothreads_not_lingering
+    @mock.sleepdeprived(module=autoscale)
+    def test_with_autoscaler_file_descriptor_safety(self):
+        # Given: a test celery worker instance with auto scaling
+        worker = self.create_worker(
+            autoscale=[10, 5], use_eventloop=True,
+            timer_cls='celery.utils.timer2.Timer',
+            threads=False,
+        )
+        # Given: This test requires a QoS defined on the worker consumer
+        worker.consumer.qos = qos = QoS(lambda prefetch_count: prefetch_count, 2)
+        qos.update()
+
+        # Given: We have started the worker pool
+        worker.pool.start()
+
+        # Then: the worker pool is the same as the autoscaler pool
+        auto_scaler = worker.autoscaler
+        assert worker.pool == auto_scaler.pool
+
+        # Given: Utilize kombu to get the global hub state
+        hub = get_event_loop()
+        # Given: Initial call the Async Pool to register events works fine
+        worker.pool.register_with_event_loop(hub)
+
+        # Create some mock queue message and read from them
+        _keep = [Mock(name='req{0}'.format(i)) for i in range(20)]
+        [state.task_reserved(m) for m in _keep]
+        auto_scaler.body()
+
+        # Simulate a file descriptor from the list is closed by the OS
+        # auto_scaler.force_scale_down(5)
+        # This actually works -- it releases the semaphore properly
+        # Same with calling .terminate() on the process directly
+        for fd, proc in worker.pool._pool._fileno_to_outq.items():
+            # however opening this fd as a file and closing it will do it
+            queue_worker_socket = open(str(fd), "w")
+            queue_worker_socket.close()
+            break  # Only need to do this once
+
+        # When: Calling again to register with event loop ...
+        worker.pool.register_with_event_loop(hub)
+
+        # Then: test did not raise "OSError: [Errno 9] Bad file descriptor!"
+
+        # Finally:  Clean up so the threads before/after fixture passes
+        worker.terminate()
+        worker.pool.terminate()
+
+    @pytest.mark.nothreads_not_lingering
+    @mock.sleepdeprived(module=autoscale)
+    def test_with_file_descriptor_safety(self):
+        # Given: a test celery worker instance
+        worker = self.create_worker(
+            autoscale=[10, 5], use_eventloop=True,
+            timer_cls='celery.utils.timer2.Timer',
+            threads=False,
+        )
+
+        # Given: This test requires a QoS defined on the worker consumer
+        worker.consumer.qos = qos = QoS(lambda prefetch_count: prefetch_count, 2)
+        qos.update()
+
+        # Given: We have started the worker pool
+        worker.pool.start()
+
+        # Given: Utilize kombu to get the global hub state
+        hub = get_event_loop()
+        # Given: Initial call the Async Pool to register events works fine
+        worker.pool.register_with_event_loop(hub)
+
+        # Given: Mock the Hub to return errors for add and remove
+        def throw_file_not_found_error(*args, **kwargs):
+            raise OSError()
+
+        hub.add = throw_file_not_found_error
+        hub.add_reader = throw_file_not_found_error
+        hub.remove = throw_file_not_found_error
+
+        # When: Calling again to register with event loop ...
+        worker.pool.register_with_event_loop(hub)
+        worker.pool._pool.register_with_event_loop(hub)
+        # Then: test did not raise OSError
+        # Note: worker.pool is prefork.TaskPool whereas
+        # worker.pool._pool is the asynpool.AsynPool class.
+
+        # When: Calling the tic method on_poll_start
+        worker.pool._pool.on_poll_start()
+        # Then: test did not raise OSError
+
+        # Given: a mock object that fakes whats required to do whats next
+        proc = Mock(_sentinel_poll=42)
+
+        # When: Calling again to register with event loop ...
+        worker.pool._pool._track_child_process(proc, hub)
+        # Then: test did not raise OSError
+
+        # Given:
+        worker.pool._pool._flush_outqueue = throw_file_not_found_error
+
+        # Finally:  Clean up so the threads before/after fixture passes
+        worker.terminate()
+        worker.pool.terminate()
+
     def test_dont_stop_or_terminate(self):
         worker = self.app.WorkController(concurrency=1, loglevel=0)
         worker.stop()
